#!/usr/bin/perl -w

use strict;
use warnings;
use RiveScript;
use JSON;

################################################################################
## Configuration Below                                                        ##
################################################################################

# Path to where your RiveScript documents can be found.
our $REPLIES = "$ENV{HOME}/XChat-RS";

# Name of a JSON file to save user info to when the plugin is unloaded.
our $DATABASE = "$ENV{HOME}/XChat-RS/userdata.json";

################################################################################
## END CONFIGURATION                                                          ##
################################################################################

# Initialize the RiveScript object.
our $rs;
our @outgoing;
our $json = JSON->new->utf8->relaxed();
&init_rivescript();

our $VERSION = '0.01';

IRC::register("xchat-rivescript.pl", $VERSION, "shutdown", "");
IRC::add_message_handler("PRIVMSG", "on_privmsg");
IRC::add_command_handler("rivescript", "on_command");

IRC::print(":: RiveScript plugin $VERSION loaded ::");

sub init_rivescript {
	undef($rs);

	# Reply folder exists?
	if (!-d $REPLIES) {
		IRC::print("Error: RiveScript reply directory $REPLIES not found!");
		return;
	}

	# Create an initialize a new RiveScript object.
	$rs = RiveScript->new();
	$rs->loadDirectory($REPLIES);
	$rs->sortReplies();

	# Reload user info from DB.
	&load_userinfo();
}

sub on_privmsg {
	my $irc_msg = shift;
	$irc_msg    =~ m/:(.*)!(\S+) PRIVMSG (.+?) :(.+?)$/i;
	my $nick    = lc($1);
	my $ident   = $2;
	my $target  = $3;
	my $message = $4;

	# Get a reply for the user.
	my $reply = $rs->reply($nick, $message);

	# If the reply contains <noreply> don't send a reply.
	if ($reply =~ /<noreply>/i) {
		return 0;
	}

	# Save changes to user data.
	&save_userinfo();

	# Queue up the reply to be sent.
	push (@outgoing, split(/<nextreply>/i, $reply));
	IRC::add_timeout_handler(1, "send_msg");

	return 0;
}

sub on_command {
	my $msg = shift || "";

	if ($msg eq "reload") {
		# Reload the brain.
		&init_rivescript();
		IRC::print(":: RiveScript reloaded! ::");
	}
	else {
		IRC::print("Usage: /rivescript <command>");
		IRC::print("Commands are:");
		IRC::print("  reload: Reload the RiveScript replies");
	}
}

sub send_msg {
	while (scalar(@outgoing)) {
		my $next = shift(@outgoing);
		IRC::command("/say $next");
	}
}

sub save_userinfo {
	# Get all user vars and dump.
	my $vars = $rs->getUservars();
	open (WRITE, ">$DATABASE") or return;
	print WRITE $json->encode($vars);
	close (WRITE);
}

sub load_userinfo {
	# Only if the DB exists.
	return unless -f $DATABASE;

	IRC::print(":: Loading RiveScript user data from $DATABASE ::");

	# Read the JSON file.
	local $/;
	open (READ, "$DATABASE");
	my $data = <READ>;
	close (READ);

	# Decode it into a data structure.
	my $vars = $json->decode($data);

	# Set all users' info.
	foreach my $nick (keys %{$vars}) {
		$rs->setUservar($nick, %{$vars->{$nick}});
	}
}

sub shutdown {
	IRC::print(":: Unloading RiveScript plugin! ::");
	undef($rs);
	return 1;
}
